# *
# *	Copyright (c) 2000 Alberto Reggiori / <alberto.reggiori@jrc.it>
# *	ISIS/RIT, Joint Research Center Ispra (I)
# *
# * NOTICE
# *
# * This product is distributed under a BSD/ASF like license as described in the 'LICENSE'
# * file you should have received together with this source code. If you did not get a
# * a copy of such a license agreement you can pick up one at:
# *
# *     http://xml.jrc.it/RDFStore/LICENSE
# *
# *
#
package Data::MagicTie::BerkeleyDB;
{
	$|=1;$::|=1;

	use BerkeleyDB;
	use Carp;

	sub sync {
		my $class = shift;
		my $id = shift;

		$class->{db}->db_sync();
	};

	sub TIEHASH {
		my ($pkg,$filename,%params) = @_;

		my $class = {};
		bless $class,$pkg;

		$class->{'dbfullfilename'} = $filename;

		$filename =~ m#(\/)(.*)(\/)#;
		$class->{'dbfilename'} = $';
		$class->{'dbdirname'} = $&;
		$class->{'db_stuff'} = {};
		umask 0;

		$params{mode} = DB_CREATE
			unless( (exists $params{mode}) && (defined $params{mode}) );

		if( $class->{'db'}  = tie ( %{ $class->{'db_stuff'} }, "BerkeleyDB::Btree", (Filename => $filename, Flags => $params{mode}, "Mode" => 0666) ) ) {
			$class->{'db'}->db_sync();
			$class->{'db_cursor'} = $class->{'db'}->db_cursor();
			return $class;
		};
		return undef;
	};


	# TIEARRAY  not yet.....
	sub TIEARRAY {
		my ($pkg,$filename,%params) = @_;

		my $class = {};
		bless $class,$pkg;

		$class->{'dbfullfilename'} = $filename;

		$filename =~ m#(\/)(.*)(\/)#;
		$class->{'dbfilename'} = $';
		$class->{'dbdirname'} = $&;
		$class->{'db_stuff'} = [];
		umask 0;

		$params{mode} = DB_CREATE
			unless( (exists $params{mode}) && (defined $params{mode}) );

		if( $class->{'db'}  = tie ( @{ $class->{'db_stuff'} }, "BerkeleyDB::Recno",( "Filename" => $filename, "Flags" => $params{mode}, "Mode" => 0666) ) ) {
			$class->{'db'}->db_sync();
			$class->{'db_cursor'} = $class->{'db'}->db_cursor();
			return $class;
		};
		return undef;
	};

	#read methods
	sub FETCH {
		my $class = shift;

		my $value;
		my $status = $class->{'db'}->db_get($_[0],$value);

		return $value;
	}

	sub inc {
		my $class = shift;

		#FETCH
		my $value;
		my $status = $class->{'db'}->db_get($_[0],$value);

		if(defined $value) {
			$value++;

			#STORE
			$status = $class->{'db'}->db_put($_[0],$value);
			$class->{'db'}->db_sync();
		};

		return $value;
	};

	sub EXISTS {
		my $class = shift;

		my $value;
                return $class->{'db'}->db_get($_[0],$value);
	};

	# we need cursors hereish....
	sub FIRSTKEY {
                my $class = shift;

		my ($key,$value);
		my $status= $class->{'db_cursor'}->c_get($key, $value, DB_FIRST);
		return $key;
        };
	
	sub NEXTKEY {
                my $class = shift;

		my ($key,$value);
		my $status= $class->{'db_cursor'}->c_get($key, $value, DB_NEXT);
		return $key;
        };

	#write methods
	sub STORE {
		my $class = shift;

		my $status = $class->{'db'}->db_put(@_);
		$class->{'db'}->db_sync();
		return $_[1];	#do not chain here
	};

	sub DELETE {
		my $class = shift;

		my $status = $class->{'db'}->db_del(@_);
		$class->{'db'}->db_sync();

                return $_[0];
	};

	sub CLEAR {
		my $class = shift;

		$class->{'db_stuff'} = {};
		$class->{'db'}->db_sync();

		return 1;
	};

	sub DESTROY {
		my $class = shift;

		if ($class->{'db'}->type() eq DB_RECNO) {
                	untie @{$class->{'db_stuff'}};
                } else {
                      	untie %{$class->{'db_stuff'}};
		};
	};

1;
}

__END__

=head1 NAME

Data::MagicTie::BerkeleyDB - This module is used by Data::MagicTie(3) to get a tied over SleepyCat BerkeleyDB files.

=head1 SYNOPSIS

see BerkeleyDB(3)

=head1 DESCRIPTION

=head1 SEE ALSO

http://www.sleepycat.com

=head1 AUTHOR

Alberto Reggiori <alberto.reggiori@jrc.it>
