# *
# *	Copyright (c) 2000 Alberto Reggiori / <alberto.reggiori@jrc.it>
# *	ISIS/RIT, Joint Research Center Ispra (I)
# *
# * NOTICE
# *
# * This product is distributed under a BSD/ASF like license as described in the 'LICENSE'
# * file you should have received together with this source code. If you did not get a
# * a copy of such a license agreement you can pick up one at:
# *
# *     http://xml.jrc.it/RDFStore/LICENSE
# *
# *

package XML::Parser::PenRDF::RDFStore;
{
use RDF::Store::Model;
use RDF::Store::FindIndex;
use Data::MagicTie; #see documentation
use Carp;

sub Init {
    my $expat = shift;

	# TIE triple database
        my $store_dir=((exists $expat->{store}) && (exists $expat->{store}->{directory})) ?
			$expat->{store}->{directory} : 
			undef;
	$expat->{triples} = {};
	$expat->{triples_index} = {};
	if (	(exists $expat->{store}->{persistent}) && 
		($expat->{store}->{persistent}==1) ) {
		$expat->{triples_db} = tie %{$expat->{triples}},
				'Data::MagicTie',
				$store_dir.'rdf/triples',(%{$expat->{store}->{options}})
			or croak "Cannot tie 'triples' database ($!)\n";
		$expat->{triples_index_db} = tie %{$expat->{triples_index}},
				'Data::MagicTie',
				$store_dir.'rdf/index',(%{$expat->{store}->{options}})
			or croak "Cannot tie 'triples index' database ($!)\n";
	};

	my $index = new RDF::Store::FindIndex($expat->{triples_index});
	$expat->{RDFStore_model} = new RDF::Store::Model(
					$expat->{PenRDF}->{nodeFactory},	
					$expat->{triples},$index,undef); 
};

sub Final {
    my $expat = shift;

	return $expat->{RDFStore_model};
};

# input: either Expat valid QNames or "assertions" (statements)
# output: "assertions" (statements)
# David Megginson saying that this is bad it is better Start/Stop Resource/Property
# anyway it should look like: Assert(subjectType,subject,predicate,objectType,object,lang)
# (see http://lists.w3.org/Archives/Public/www-rdf-interest/1999Dec/0045.html)
sub Assert {
	my ($expat,$st) = @_;

	print $st->toString,"\n"
		if(defined $expat->{store}->{seevalues});

	$expat->{RDFStore_model}->add($st);
};

# we might use this callback for XSLT tansofrmations of xml-blobs :)
sub Start_XML_Literal {
	my ($expat,$tag,%attlist) = @_;

	return $expat->recognized_string; # cool eh ;-)
};

sub Stop_XML_Literal {
	my ($expat,$tag) = @_;

	return $expat->recognized_string;
};

sub Char_Literal {
	my ($expat,$literal_text) = @_;

	#return $literal_text;
	return $expat->recognized_string;
};

1;
}

package XML::Parser::PenRDF::Mysql; # any ? ;-)

__END__

=head1 NAME

XML::Parser::PenRDF::RDFStore - This module is an extension of XML::Parser::PenRDF(3) that actually use ab RDF API modules to ingest records into an RDFStore database.

=head1 SYNOPSIS

use XML::Parser::PenRDF;
use XML::Parser::PenRDF::RDFStore;
use RDF::Store::NodeFactory;
my $p=new XML::Parser::PenRDF(	ErrorContext => 2,
                                Style => 'XML::Parser::PenRDF::RDFStore',
                                NodeFactory     => new RDF::Store::DBMS::NodeFactory()
                                );

if(defined $ENV{GATEWAY_INTERFACE}) {
        print "Content-type: text/html

";
        $p->parsefile($ENV{QUERY_STRING});
} else {
        my $input = shift;
        if($input =~ /^-/) {
                $p->parse(*STDIN);
        } else {
                $p->parsefile($input);
        };
};

=head1 DESCRIPTION

In the samples directory of the distribution you can find a set of a sample scripts to play with :)

=head1 BUGS

None known yet

=head1 SEE ALSO

XML::Parser::PenRDF(3), Data::MagicTie(3) RDF::Store(3) DBMS(3)

=head1 AUTHOR

Alberto Reggiori <alberto.reggiori@jrc.it>
